'''
Calculate which of the prediction points have overlap with AJDs. These will be
excluded from consideration in picking which points for the AJD prediction model.
'''

import numpy as np
import pandas as pd
import geopandas as gpd
import pygeos as pg
from glob import glob
from tqdm import tqdm
import multiprocessing as mp

footprints = '/home/sgreenhill/wotus/satellite_imagery/tile_footprints'
naip = gpd.read_file(f'{footprints}/NAIP_tiles.shp')
# naip['pointid'] = naip.pointid.astype(int)
naip.crs = 'WGS84'
naip_polygons = pg.from_shapely(naip.geometry)

grid_path = '/home/sgreenhill/wotus/prediction_points/grid_points/'
paths = glob(f'{grid_path}/grid_tiles_*')

def get_ajd_overlap(p):
    shp = gpd.read_file(p)
    grid_polygons = pg.from_shapely(shp.geometry)
    shp['intersects_ajds'] = [any(pg.intersects(g, naip_polygons)) for g in grid_polygons]
    shp[['prdctn_', 'intersects_ajds']].to_csv(f'{p}/point_ajd_overlap.csv', index=False)
    return f'Done with {p}'

with mp.Pool(16) as pool:
    done = list(tqdm(pool.imap(get_ajd_overlap, paths), total=len(paths)))