library(raster)
library(tigris)
library(rgdal)

path <- "/Users/hannahdruckenmiller/Desktop/cwa/"

# Continental US is sample area 
states <- states(cb=T)
states <- as(states, 'Spatial')
states <- subset(states, !(STATEFP %in% c("02", "15", "60", "66", "69", "72", "78")))

extent_states <- extent(states)
extent_states[1] <- floor(extent_states[1]*10)/10
extent_states[2] <- ceiling(extent_states[2]*10)/10
extent_states[3] <- floor(extent_states[3]*10)/10
extent_states[4] <- ceiling(extent_states[4]*10)/10

# Make raster covering this area with resolution of 0.1 degrees (~10km)
grid <- raster(xmn = extent_states[1], xmx = extent_states[2], 
               ymn = extent_states[3], ymx = extent_states[4],
               res = 0.1)
grid[] <- 1
grid <- mask(grid, states)
plot(grid)

grid_points <- as.data.frame(rasterToPoints(grid))
grid_points$xmin <- grid_points$x - 0.05
grid_points$xmax <- grid_points$x + 0.05
grid_points$ymin <- grid_points$y - 0.05
grid_points$ymax <- grid_points$y + 0.05
grid_points$grid_cell <- 1:nrow(grid_points)

# Save sampling grid info 
grid <- rasterize(grid_points[,c("x", "y")], grid, grid_points$grid_cell)
writeRaster(grid, paste0(path, "sample_grid_raster"), format="GTiff")
grid_points <- grid_points[,c("grid_cell", "x", "y", "xmin", "xmax", "ymin", "ymax")]
write.csv(grid_points, paste0(path, "sample_grid_raster.csv"), row.names = F)

# Randomly sample 50 points in each grid cell 
points <- as.data.frame(matrix(NA, 50, 4))
colnames(points) <- c("grid_cell", "process_order", "lon", "lat")

for(c in 1:nrow(grid_points)){
  print(c)
  x <- runif(50, grid_points$xmin[c], grid_points$xmax[c])
  y <- runif(50, grid_points$ymin[c], grid_points$ymax[c])
  points$grid_cell <- grid_points$grid_cell[c]
  points$process_order <- 1:50
  points$lon <- x
  points$lat <- y
  if(c==1){data <- points}else{data <- rbind(data, points)}
}

data$prediction_id <- 1:nrow(data)

# save points 
write.csv(data, past0(path, "sample_grid_points.csv"), row.names = F)

# save points as spatial object
sp <- SpatialPoints(data[,c("lon", "lat")])
proj4string(sp) <- proj4string(grid)
sp$prediction_id <- data$prediction_id
sp$grid_cell <- data$grid_cell
sp$process_order <- data$process_order

writeOGR(sp, paste0(path, "sample_grid_points_sp"), "sample_grid_points", drive = "ESRI Shapefile")

