rm(list=ls())

library(sp)
library(raster)
library(rgdal)
library(readr)
library(dplyr)
library(sf)
library(doParallel)

### MAKE SHAPEFILE WITH NAIP FOOTPRINTS for 4 million prediction points 

path <- "/shares/lab/hannah/wetlands/cwa/"

# Batched bounding boxes for NAIP images, retrieved from Descartes Labs 
for(t in 0:4){
  print(t)
  data <- read_csv(paste0(path, "prediction_points_sample_grid_points_NAIP_tiles_", t, ".csv"))
  data <- data[,c("prediction_id", "lon", "lat", "geometry")]
  if(t==0){tiles <- data}else{tiles <- rbind(tiles, data)}
}
rm(data)

# Metadata for prediction points 
grid <- read_csv(paste0(path, "sample_grid_points.csv"))
tiles <- left_join(tiles, grid[,c("prediction_id", "process_order")])
data <- tiles
rm(tiles, grid)

registerDoParallel(12)

# Create shapefiles from boudning boxes 
foreach(order=2:50) %dopar% {

  tiles <- subset(data, process_order==order)
  
  for(i in 1:nrow(tiles)){
    if(i%%100==0){print(paste(order, ":", i, "of", nrow(tiles)))}
    poly <- tiles$geometry[i]
    points <- unlist(strsplit(poly, ","))
    x <- c()
    y <- c()
    for (p in 2:5){
      point <- substr(points[p],2,nchar(points[p]))
      point <- unlist(strsplit(point, " "))
      lon <- point[1]
      lat <- point[2]
      if(p==5){lat <- substr(lat, 1, nchar(lat)-2)}
      x <- append(x, as.numeric(lon))
      y <- append(y, as.numeric(lat))
      }
    xys <- as.data.frame(cbind(x, y))
    xys$prediction_id <- tiles$prediction_id[i]
    if(i==1){sp <- xys}else{sp <- rbind(sp, xys)}
  }
  points = st_as_sf(sp, coords=c("x","y"))
  polys = points %>% dplyr::group_by(prediction_id) %>% dplyr::summarise() %>% st_cast("POLYGON") %>% st_convex_hull() 
  polys$lon <- tiles$lon
  polys$lat <- tiles$lat
  polys$process_order <- tiles$process_order
  st_write(polys, paste0(path, "grid_tiles_", order), 
         paste0("grid_tiles_", order),
         driver = "ESRI Shapefile", append = F)
}

