rm(list=ls())

library(dplyr)
library(naniar)
library(readr)

path <- "/shares/lab/hannah/wetlands/cwa/prediction_point_ints/"

### SET DATA SET ####

# 4 million prediction poitns 
id <- "prdctn_"
save_path <- paste0(path, "prediction_point_intersections.csv")

# AJDs
#id <- "id"
#save_path <- paste0(path, "ajd_point_intersections.csv")

#####################

# Load files (generated in batches, need to be pasted back together)
files <- list.files(path)
length(files)

for(file in files){
  print(file)
  data <- read_csv(paste0(path, file))
  if(nrow(data) > 0){
  data <- data[,c(id, "nwi_type", "fcode", "navigable_water",
                   "iclus_2020", "iclus_2030")]
  if(file==files[1]){df <- data}else{df <- rbind(df, data)}
  print(nrow(df))
  }
}

colnames(df)[1] <- "prediction_id"

# NWI type, numbers correspond to GEE values, legend in printed look up table 
df$nwi_type[df$nwi_type==1] <- "Lake"
df$nwi_type[df$nwi_type==2] <- "Other"
df$nwi_type[df$nwi_type==3] <- "Riverine"
df$nwi_type[df$nwi_type==4] <- "Freshwater Pond"
df$nwi_type[df$nwi_type==5] <- "Freshwater Emergent Wetland"
df$nwi_type[df$nwi_type==6] <- "Freshwater Forested/Shrub Wetland"
df$nwi_type[df$nwi_type==7] <- "Estuarine and Marine Wetland"
df$nwi_type[df$nwi_type==8] <- "Estuarine and Marine Deepwater"
df$nwi_type[df$nwi_type==10] <- "Riverine"

# ICLUS codes at bottom of script 
df$iclus_2020_development_intensity <- NA
df$iclus_2020_development_intensity[df$iclus_2020 %in% c(0, 1, 2, 3, 4, 5, 6, 7, 8)] <- 1
df$iclus_2020_development_intensity[df$iclus_2020 %in% c(9, 10, 12, 13)] <- 2
df$iclus_2020_development_intensity[df$iclus_2020 %in% c(11, 14,15,16,17,18)] <- 3

df$iclus_2030_development_intensity <- NA
df$iclus_2030_development_intensity[df$iclus_2030 %in% c(0, 1, 2, 3, 4, 5, 6, 7, 8)] <- 1
df$iclus_2030_development_intensity[df$iclus_2030 %in% c(9, 10, 12, 13)] <- 2
df$iclus_2030_development_intensity[df$iclus_2030 %in% c(11, 14,15,16,17,18)] <- 3

df$iclus_growth <- df$iclus_2030_development_intensity > df$iclus_2020_development_intensity

# Any NWI or NHD? 
df$nwi <- !is.na(df$nwi_type)
df$nhd <- !is.na(df$fcode)

# Save data 
colnames(df) <- c("prediction_id", "nwi", "nwi_wetland_type", "nhd", "nhd_fcode", "iclus_growth")
write.csv(df, save_path, row.names = F)

# Print some summary statistics 
print(paste("NWI polygons", sum(df$nwi)))
print(paste("NHD flowlines", sum(df$nhd)))
print(paste("Navigable waters", sum(df$navigable_water)))
print(paste("Navigable waters + NWI", sum(df$navigable_and_nwi)))
print(paste("ICLUS growth areas", sum(df$iclus_growth, na.rm = T)))
print(paste("Total points", nrow(df)))

print(paste("Lake", sum(df$nwi_wetland_type=="Lake", na.rm = T)))
print(paste("Freshwater Pond", sum(df$nwi_wetland_type=="Freshwater Pond", na.rm = T)))
print(paste("Riverine", sum(df$nwi_wetland_type=="Riverine", na.rm = T)))
print(paste("Freshwater Emergent Wetland", sum(df$nwi_wetland_type=="Freshwater Emergent Wetland", na.rm = T)))
print(paste("Freshwater Forested/Shrub Wetland", sum(df$nwi_wetland_type=="Freshwater Forested/Shrub Wetland", na.rm = T)))
print(paste("Estuarine and Marine Wetland", sum(df$nwi_wetland_type=="Estuarine and Marine Wetland", na.rm = T)))
print(paste("Estuarine and Marine Deepwater", sum(df$nwi_wetland_type=="Estuarine and Marine Deepwater", na.rm = T)))
print(paste("Other", sum(df$nwi_wetland_type=="Other", na.rm = T)))

print(paste("Intermittent stream", sum(df$nhd_fcode==46003, na.rm = T)))
print(paste("Perennial stream", sum(df$nhd_fcode==46006, na.rm = T)))
print(paste("Coastline", sum(df$nhd_fcode==56600, na.rm = T)))


# ICLUS codes 
# 0 = natural water
# 1 = reserviors, canals
# 2 = wetlands 
# 3 = recreation/conservation
# 4 = timber 
# 5 = grazing
# 6 = pasture
# 7 = cropland
# 8 = mining, barren
# 9 = parks, golf
# 10 = exurban, low 
# 11 = exurban, high
# 12 = suburban
# 13 = urban low
# 14 = urban high 
# 15 = commerical
# 16 = industrial
# 17 = institutional 
# 18 = transportation 

