# Create rule dummy layers for training a pooled model across all rules
pacman::p_load(pbmcapply, tidyverse, glue)

source('/home/sgreenhill/repos/WOTUS/new_imagery/data/rasterization_functions.R')

split = fread('/home/sgreenhill/wotus/misc/train_test_split_naip_only.csv')

split = split[rule != '']
split[, rule_numeric := as.numeric(factor(rule))]

split[, pointid_str := str_pad(pointid, 6, pad='0')]

make_rule_fe = function(
    this_pointid, footprints=naip_footprints, directory='NAIP_rule_dummies',
    dim=512, overwrite=TRUE, return_raster=FALSE, val='rule_val'
) {
    dir = glue('/home/sgreenhill/wotus/satellite_imagery/{directory}')
    file = glue('{dir}/{this_pointid}.tif')
    if (file.exists(file) & !overwrite) {
        return(glue('{this_pointid} already run.'))
    }

    # fill a raster with the district number
    footprint = get_footprint(this_pointid, footprints, dim=dim)

    if (val == 'rule_val') {
        rule_val = split[pointid_str==this_pointid, rule_numeric]
    } else {
        rule_val = val
    }
    

    footprint = setValues(footprint, rule_val)

    if (!dir.exists(dir)) {
        dir.create(dir)
    }
    writeRaster(
        footprint,
        file,
        overwrite=TRUE
    )

    if (return_raster) {
        return(footprint)
    } else {
        return(glue('Successfully run {this_pointid}.'))
    }
}

done = pbmclapply(split$pointid_str, make_rule_fe, mc.cores=64, ignore.interactive=TRUE)

# for the test set only, make points for all 3 rules -- for prediction of test set
# point on each of the three rules
test = split[split_group == 'test']
rules = c('Rapanos', 'CWR', 'NWPR')
for (i in seq_along(rules)) {
    rule = rules[i]
    rule_val = i
    done = pbmclapply(test$pointid_str, make_rule_fe, 
    directory=glue('NAIP_rule_dummies/{rule}'), val=rule_val,
    mc.cores=96, ignore.interactive=TRUE)
}