# Create state district dummy layers for each NAIP footprint.

pacman::p_load(sf, glue, data.table, pbmcapply, terra, dplyr)

source('/home/sgreenhill/repos/WOTUS/new_imagery/data/rasterization_functions.R')

ajds = fread('/home/sgreenhill/wotus/AJD/jds_wet_dry_season_clean_v2.csv')

# encode numerically, going from west to east, snaking north to south
ajds[, state_numeric := as.numeric(
    factor(
        state,
        levels=c(
            'Washington', 'Oregon', 'California', 'Arizona', 'Nevada', 
            'Utah', 'Idaho', 'Montana', 'Wyoming', 'Colorado', 'New Mexico',
            'Texas', 'Oklahoma', 'Kansas', 'Nebraska', 'South Dakota', 
            'North Dakota', 'Minnesota', 'Iowa', 'Missouri', 'Arkansas', 'Louisiana',
            'Mississippi', 'Tennessee', 'Illinois', 'Wisconsin', 'Michigan',
            'Indiana', 'Kentucky', 'Alabama', 'Florida', 'Georgia', 'South Carolina',
            'North Carolina', 'West Virginia', 'Ohio', 'Pennsylvania', 'New York', 
            'Vermont', 'Maine', 'New Hampshire', 'Massachusetts', 'Connecticut',
            'Rhode Island', 'New Jersey', 'Delaware', 'Maryland', 
            'District of Columbia', 'Virginia'
        )
    )
)
]

ajds[, pointid_str := str_pad(pointid, 6, pad='0')]

make_state_fe = function(
    this_pointid, footprints=naip_footprints, directory='NAIP_states', dim=512,
    overwrite=FALSE
) {
    file = glue('/home/sgreenhill/wotus/satellite_imagery/{directory}/{this_pointid}.tif')
    if (file.exists(file) & !overwrite) {
        return(glue('{this_pointid} already run.'))
    }

    # fill a raster with the district number
    footprint = get_footprint(this_pointid, footprints, dim=dim)

    state_val = ajds[pointid_str==this_pointid, state_numeric]

    footprint = setValues(footprint, state_val)

    writeRaster(
        footprint,
        file,
        overwrite=overwrite
    )

    return(glue('Successfully run {this_pointid}.'))
}

pts = ajds$pointid_str
done=pbmclapply(pts, make_state_fe, mc.cores=8, ignore_interactive=TRUE)