/*
Intersect 4 million points with NHD, NWI, and ICLUS
This script can be run in the Google Earth Engine Code Editor by following
this link: https://code.earthengine.google.com/eb0e5325816cc1759a4b3c3ffd4cdf1a
Associated assets are public.
*/ 



var nhd = ee.Image("projects/nwi-hannahdruckenmiller/assets/NHD_image_full_2"),
    geometry = 
    /* color: #98ff00 */
    /* displayProperties: [
      {
        "type": "rectangle"
      },
      {
        "type": "marker"
      }
    ] */
    ee.Geometry({
      "type": "GeometryCollection",
      "geometries": [
        {
          "type": "Polygon",
          "coordinates": [
            [
              [
                -71.58584270470517,
                42.01050976743218
              ],
              [
                -71.58584270470517,
                42.008261690817626
              ],
              [
                -71.57701287262815,
                42.008261690817626
              ],
              [
                -71.57701287262815,
                42.01050976743218
              ]
            ]
          ],
          "geodesic": false,
          "evenOdd": true
        },
        {
          "type": "Point",
          "coordinates": [
            -71.57850175560215,
            42.00863097137067
          ]
        }
      ],
      "coordinates": []
    }),
    iclus20 = ee.Image("projects/nwi-assets/assets/ICLUS_v2_1_1_land_use_conus_2020_ssp2_rcp45_giss_e2_r"),
    iclus30 = ee.Image("projects/nwi-assets/assets/ICLUS_v2_1_1_land_use_conus_2030_ssp2_rcp45_giss_e2_r"),
    nwi = ee.Image("projects/nwi-hannahdruckenmiller/assets/NWI_image_30m_US");
    
// All prediction points 
var points = ee.FeatureCollection("projects/nwi-assets/assets/sample_grid_points")

// Add state boundaries 
var states = ee.FeatureCollection("TIGER/2016/States")

// For each state 
var state_fips = ['01' , '04' , '05' , '06' , '08' , '09' , '10' , '11' , '12' , '13' , '16' , '17' , '18' , '19' , '20' , '21' , '22' , '23' , '24' , '25' , '26' , '27' , '28' , '29' , '30' , '31' , '32' , '33' , '34' , '35' , '36' , '37' , '38' , '39' , '40' , '41' , '42' , '44' , '45' , '46' , '47' , '48' , '49' , '50' , '51' , '53' , '54' , '55' , '56']
for (var i = 0;i<state_fips.length;i++){

var fips = state_fips[i]
var state = states.filter(ee.Filter.eq('STATEFP', fips));

// NWI 
var nwi_state = nwi.clip(state)
nwi_state = nwi_state.round()
nwi_state = nwi_state.rename('nwi_type')

// NWI 500m buffer
var nwi_any_state = nwi_state.remap([0,1,2,3,4,5,6,7,8,9,10], 
[1,1,1,1,1,1,1,1,1,1,1])
nwi_any_state = nwi_any_state.rename('nwi_buffer')

var nwi_buffer_state = nwi_any_state.focal_max({
radius: 500,
kernelType: 'circle',
units:'meters',
iterations: 1
})


// NHD
var nhd_state = nhd.clip(state)

var points_state = points.filterBounds(state)

var get_buffer = function(feature){
  var buff = feature.buffer(5)
  return buff 
}

var points_buffer_state = points_state.map(get_buffer)

var get_intersections = function(feature){
var point = feature

var nwi_int = nwi_state.reduceRegion({
  reducer: ee.Reducer.median(),
  geometry: point.geometry(),
  scale: 10,
});

var buffer_int = nwi_buffer_state.reduceRegion({
  reducer: ee.Reducer.max(),
  geometry: point.geometry(),
  scale: 10,
});

var nhd_int = nhd_state.select('fcode').reduceRegion({
  reducer: ee.Reducer.first(),
  geometry: point.geometry(),
  scale: 10,
});


var iclus20_int = iclus20.select('b1').reduceRegion({
  reducer: ee.Reducer.median(),
  geometry: point.geometry(),
  scale: 10,
});

var iclus30_int = iclus30.select('b1').reduceRegion({
  reducer: ee.Reducer.median(),
  geometry: point.geometry(),
  scale: 10,
});

var nav_int = rivers.reduceRegion({
  reducer: ee.Reducer.max(),
  geometry: point.geometry(),
  scale: 10,
});

var feat = point.set('nwi_type', nwi_int.get('nwi_type'));
feat = feat.set('nwi_buffer', buffer_int.get('nwi_buffer'));
feat = feat.set('fcode', nhd_int.get('fcode'));
feat = feat.set('navigable_water', nav_int);
feat = feat.set('iclus_2020', iclus20_int.get('b1'));
feat = feat.set('iclus_2030', iclus30_int.get('b1'));
return(feat)
}

var data = points_buffer_state.map(get_intersections)

Export.table.toDrive({collection: data,   
                  description: 'prediction_points_' + fips,   
                   fileFormat: 'CSV', 
                    folder: 'prediction_point_ints' });


}

