rm(list=ls())

library(sp)
library(rgdal)

path <- "/Users/hannahdruckenmiller/Desktop/cwa/"

tasks <- c("ajd", "pjd", "sackett", "prediction", "navigable")

for(task in tasks){

#######################

# AJD 
  if(task=="ajd"){
points <- read.csv(paste0(path, "AJD_jds_wet_dry_season_clean_v2.csv"))
id_name <- "pointid"
save_file <- "ajd_metadata.csv"
points <- SpatialPointsDataFrame(points[,c("longitude", "latitude")], data = as.data.frame(points[,id_name]))
  }
  
# PJDS 
  if(task=="pjd"){
points <- read.csv(paste0(path, "prediction_points_pjd_101000_NAIP_tiles.csv"))
id_name <- "prediction_id"
save_file <- "pjd_metadata.csv"
points <- SpatialPointsDataFrame(points[,c("lon", "lat")], data = as.data.frame(points[,id_name]))
  }
  
# SACKETT 
  if(task=="sackett"){
points <- readOGR(paste0(path, "Sackett_sp/Sackett_sp.shp"))
id_name <- "id"
save_file <- "sackett_metadata.csv"
  }
  
# 4 million 
  if(task=="prediction"){
points <- readOGR(paste0(path, "sample_grid_points_sp/sample_grid_points.shp"))
id_name <- "FID"
save_file <- "prediction_metadata.csv"
}

# Navigable waterways 
  if(task=="navigable"){
points  <- as.data.frame(readOGR(paste0(path, "grid_tiles_nav_water.shp")))
id_name <- "prdctn_"
save_file <- "navigable_metadata.csv"
points <- SpatialPointsDataFrame(points[,c("lon", "lat")], data = as.data.frame(points[,id_name]))
}

#######################

colnames(points@data)[1] <- "id"
proj4string(points) <- "+proj=longlat +ellps=WGS84 +no_defs"

states <- tigris::states(cb = T)
states <- as(states, 'Spatial')
states <- subset(states, !(STATEFP %in% c("02", "15", "60", "66", "69", "72", "78")))
states <- spTransform(states, proj4string(points))

hq <- readOGR(paste0(path, "USACE_Offices/USACE_Offices.shp"))
hq <- subset(hq, STATE %in% unique(states$STUSPS))
hq <- spTransform(hq, proj4string(points))

bound <- readOGR(paste0(path, "USACE_Regulatory_Boundary/USACE_Regulatory_Boundary.shp"))
bound  <- spTransform(bound, proj4string(points))

over <- over(points, states)
points$state <- over$STATEFP

over <- over(points, bound)
points$district <- over$DIST_ABBR


points <- subset(points, !is.na(state) & !is.na(district))

districts <- unique(points$district)

over <- over(hq, bound)
hq$DIST_ABBR <- over$DIST_ABBR


for(district in districts){
  print(district)
  sub <- points[points$district==district,]
  b <- subset(hq, DIST_ABBR==district)
  if(nrow(b)==0){b <- hq}
  sp <- SpatialPoints(b[,c("X", "Y")])
  proj4string(sp) <- proj4string(bound)
  dist <- geosphere::distm(sub, sp)
  sub$distHQ <- apply(dist, 1, min)
  if(district==districts[1]){data <- sub}else{data <- rbind(data, sub)}
  if(task=="Sackett"){data <- sub}
}

library(dplyr)
data <- as.data.frame(data)
data <- arrange(data, id)
data <- data[,c("id", "state", "district", "distHQ")]

write.csv(data, paste0(path, save_file), row.names = F)

}
