# `WOTUS-ML/data/inputs`
Code in this directory is used to download and process the inputs data for all the deep learning models.
There are two subdirectories:
* `1_access/`: code for accessing (i.e. downloading) input data. Files include:
    * `get_input_layers_ajd_pjd_sackett_nav.py`: Download input layers for the AJDs, PJDs, Sackett prediction points, and navigable water prediction points using Google Earth Engine.
    * `get_4million_points.R`: Randomly select 4 million prediction points in CONUS
    * `make_NAIP_footprints.R`: get the spatial footprints of the NAIP imagery (and all other inputs) for the 4 million points.
    * `get_input_layers_4million`: Download input layers from the 4 million points using Google Earth Engine.
* `2_process/`: code for processing input data once dowloaded. Files include:
    * `get_intersections_GEE`, `clean_intersections.R`: Compute and clean intersections between prediction points and NWI, NHD, ICLUS, and other spatial layers of interest
    * `get_metadata.R`: Given points, compute metadata, including which state and ACE district the point is in and the distance between the point and the ACE district headquarters.
    * Scripts for creating ordinal and one-hot layers:
        * `create_district_and_prediction_ordinal_layers.R`: Create the the district ordinal layer for the AJD data and all the ordinal and one-hot layers for the prediction points
        * `create_rule_layer.R`: Create the rule ordinal layer for the AJD data
        * `create_state_layer.R`: Create the state ordinal layer for the AJD data
        * `rasterization_functions.R`: Define utilities for the above.
    * Notebooks for calculating layer means and standard deviations for normalization: `calculate_layer_mean_sd.ipynb`, `calculate_nhd_mean_sd.ipynb`. `data_utils.py` defines utilities for efficiently computing layer means and standard deviations, taking advantage of mulit-threading. 
    * Other file names are self-explanatory; see individual scripts and notebooks for further details.
* `3_filter/`: In rare cases, input layers are not available or are corrupted for a point. This can occur if the point is in an area not covered by NAIP or another input layer. Examples include offshore points, points that are outside national borders, and points in classified military installations such as Area 51. Notebooks in this directory identify these points and save lists of them so they can be skipped when loading data to feed into models.

