'''
Utilities for running diagnostics on DL model outputs.
'''

import pandas as pd
import numpy as np

rule_dict = {'Rapanos': '1986/88', 'CWR': 'CWR', 'NWPR':'NWPR'}
base_path = '/home/sgreenhill/repos/WOTUS/new_imagery/experiments'

def group_accuracy(group, preds_col='predictions', labels_col='labels'):
    return sum(group[preds_col] == group[labels_col]) / len(group)


def load_acc(path, n_ma=3):
    split = pd.read_csv('/home/sgreenhill/wotus/misc/train_test_split.csv')
    
    sp = path.split('_')
    rule = rule_dict[sp[1]]
    district = sp[2]
    colnames = ['predictions', 'labels', 'preds_batch', 'pointid', 'epoch']
    train_preds = pd.read_csv(f'{base_path}/{path}/train_preds.csv')
    if train_preds.columns[0] != 'predictions':
        train_preds = pd.read_csv(f'{base_path}/{path}/train_preds.csv',
                                  names=colnames)
        val_preds = pd.read_csv(f'{base_path}/{path}/val_preds.csv',
                                names=colnames)
    else:
        val_preds = pd.read_csv(f'{base_path}/{path}/val_preds.csv')
    
    grouper = 'epoch'
    train_acc = pd.DataFrame(
        train_preds.groupby(grouper).apply(group_accuracy),
        columns=['accuracy']
    ).reset_index()

    val_acc = pd.DataFrame(
        val_preds.groupby(grouper).apply(group_accuracy),
        columns=['accuracy']
    ).reset_index()
    
    # calculate naive baseline acc
    baseline_accs = {}
    Ns = {}
    for fold in ['train', 'val']:
        if district != 'small':
            this_split = split.loc[(split.ace_district == district) & 
                                (split.rule == rule) &
                                (split.split_group == fold)]
            
            N = len(this_split)
            n = len(this_split.loc[this_split.wotus == 'Yes'])
            
            Ns[fold] = [N]

            baseline = max(n / N, 1 - n / N)

            baseline_accs[fold] = [baseline]
        else:
            # Not yet implemented for this
            Ns[fold] = np.nan
            baseline_accs[fold] = np.nan
    
    # get average accuracy over best n_avg consecutive epochs
    if n_ma > 1:
        best_val_acc = val_acc.accuracy.rolling(window=n_ma).mean().max()
        best_train_acc = train_acc.accuracy.rolling(window=n_ma).mean().max()
    else:
        best_val_acc = val_acc.accuracy.max()
        best_train_acc = train_acc.accuracy.max()
    
    ret = pd.DataFrame({
        'ace_district': [district], 
        'rule': [rule], 
        'baseline_train_acc': baseline_accs['train'],
        'baseline_val_acc': baseline_accs['val'],
        'best_train_acc': [best_train_acc], 
        'best_val_acc': [best_val_acc],
        'n_train': Ns['train'],
        'n_val': Ns['val']
    })
    
    # get best epoch
    if n_ma == 1:
        best_val_epoch = val_acc.loc[val_acc.accuracy == best_val_acc, 'epoch'].values
        best_train_epoch = val_acc.loc[train_acc.accuracy == best_train_acc, 'epoch'].values
        if len(best_val_epoch) > 1:
            # if multiple best, just take the first
            best_val_epoch = best_val_epoch[:1]
            best_train_epoch = best_train_epoch[:1]
        ret['best_val_epoch'] = best_val_epoch
        ret['best_train_epoch'] = best_train_epoch
    
    return ret