'''
Predict AJD model on test set
'''

import sys
import os
src_path = '/home/sgreenhill/repos/WOTUS/new_imagery/src'
sys.path.append(src_path)
os.chdir(src_path)
import torch_utils
import models
import data
import train
import torch
import torch.nn as nn
import torch.optim as optim
from torch.multiprocessing import cpu_count
import numpy as np
import pandas as pd
from tqdm import tqdm
import multiprocessing as mp

# load the model
model_dir = '/home/sgreenhill/repos/WOTUS/new_imagery/analysis/ajd/'
model_name = 'model_results/'

chkpt_file = model_dir + model_name + 'best.pth.tar'
model = models.resnet18(in_channels=31, n_classes=2)
learning_rate = 1e-3
regularization = 1e-4

torch.cuda.set_device(0)

chkpt = torch_utils.load_checkpoint(chkpt_file, model, device=torch.device('cuda:0'))

model = nn.DataParallel(model)
model.eval()

batch_size = 1
num_cores = int(cpu_count())

path = '/home/sgreenhill/wotus'

split_path = f'{path}/misc/ajd_train_test_split.csv'
split = pd.read_csv(split_path, low_memory=False)
split_slug = 'naip_ajd'

# drop pointids where data is missing
split = data.drop_missings(split, drop_S2=False)
# drop pred points where NHD is corrupted
missing_new_nhd = np.load(f'{path}/prediction_points/new_nhd_corrupted_points.p.npy', allow_pickle=True)
split['pred_id'] = np.where(split.id.str.startswith('pred'), split.id.str.slice(5).astype(int), np.nan)

# Drop any missing or corrupted points
# corrupted files from the new NHD data
pred_dir = f'/home/sgreenhill/wotus/prediction_points/'
missing_new_nhd = np.load(f'{pred_dir}/corrupted_new_nhd_points.npy', allow_pickle=True)
# files entirely missing from HQ dist, rule, state, district FE layers
fe_missings = np.load(f'{pred_dir}/missing_fe_pts.npy', allow_pickle=True)
# corrupted rule, HQ dist, state, ace districts files
rule_missings = np.array([])
for rule in ['Rapanos', 'CWR', 'NWPR']:
    rule_missings = np.concatenate(
        [rule_missings,
         np.load(f'{pred_dir}/rules/rule_drops_{rule}.npy', allow_pickle=True)
         ]
    )
districts_missings = np.load(f'{pred_dir}/corrupted_ace_district_points.npy', allow_pickle=True)
hq_dist_missings = np.load(f'{pred_dir}/corrupted_hq_dist_points.npy', allow_pickle=True)
state_missings = np.load(f'{pred_dir}/corrupted_state_points.npy', allow_pickle=True)

missings = np.array([])
naip_missings = np.array([])

for grid in range(1, 51):
    missings = np.concatenate([
        missings,
        np.load(f'{pred_dir}/grid_{grid}/missing_pts.npy', allow_pickle=True)
    ])
    naip_missings = np.concatenate([
        naip_missings, 
        np.load(f'{pred_dir}/grid_{grid}/naip_drops.npy', allow_pickle=True)
    ])

corrupted_nlcd = np.load(f'{pred_dir}/corrupted_nlcd_points.npy')

missings = np.concatenate(
    [
        missings, 
        missing_new_nhd, 
        fe_missings, rule_missings, districts_missings, 
        hq_dist_missings, state_missings, 
        naip_missings,
        corrupted_nlcd
    ]
)


split = split.loc[~split.pred_id.isin(missings.astype(int))]

# prep the data separately, then combine 
ajd_split = split.loc[split.ajd == 1]
pred_split = split.loc[split.ajd == 0]

test = split.loc[split.split_group == 'test']

ajd_names, _, _, _, ajd_labels = data.make_split_dicts(
    ajd_split, wotus=False, ajd=True, verbose=False
)

folds = ['test']
normalize_path='/home/sgreenhill/repos/WOTUS/new_imagery/analysis/wotus/main_model/train/layer_mean_sd'
ajd_dataset = {x: data.NAIPDataset(
    root_dir='/home/sgreenhill/wotus/satellite_imagery',
    normalize=True, normalize_path=normalize_path,
    image_names=ajd_names[x], fold=x, ajd_labels=ajd_labels[x], wotus=False, ajd=True,
    n_naip_bands=4, naip=True, image_dir='AJD_inputs/NAIP', nwi=True, 
    nhd=True, nhd_dir='../prediction_points/NHD_new', 
    dem=True, ecoregion=True, 
    nlcd=True, prism=True, gnatsgo=True, hq_dist=True, district_dummies=True, 
    regrid=False, rule_dummies=False, states=True,
    save=False, load_saved=True,
    save_dir=f'/home/sgreenhill/wotus/satellite_imagery/pipeline_ready/ajd_new_nhd'
) for x in folds}

# need to loop over all 50 grids here
pred_datasets = []
for grid in range(1, 51):
    this_split = pred_split.loc[pred_split.prcss_r == grid]
    
    if this_split.shape[0] == 0:
        continue

    pred_names, _, _, _, pred_labels =  data.make_split_dicts(
        this_split, wotus=False, ajd=True, verbose=False
    )

    pred_datasets += [{x: data.NAIPDataset(
        image_names=pred_names[x], fold=x, ajd_labels=pred_labels[x],
        normalize=True, normalize_path=normalize_path,
        wotus=False, ajd=True, n_naip_bands=4, 
        naip=True, nwi=True, nhd=True, dem=True, ecoregion=True, nlcd=True, prism=True,
        gnatsgo=True, hq_dist=True, 
        district_dummies=True, regrid=False, rule_dummies=False, states=True,
        root_dir= f'{pred_dir}/grid_{grid}',
        image_dir = f'NAIP/', image_prefix='NAIP_',
        nwi_dir='NWI', nhd_dir='../NHD_new_prediction/', dem_dir='DEM', 
        ecoregion_dir = 'Ecoregion', nlcd_dir='NLCD', prism_dir='PRISM', 
        prism_ppt_dir='PPT', gnatsgo_dir='gNATSGO', 
        dem_prefix='DEM_', ecoregion_prefix='Ecoregion_',
        gnatsgo_prefix='gNATSGO_',
        nlcd_prefix='NLCD_', prism_prefix='PRISM_', 
        prism_ppt_prefix='PPT_', gnatsgo_stacked=True,
        hq_dist_dir='../HQ_dist/', hq_dist_prefix='',
        rule_dummies_dir=None, rule_dummies_prefix='',
        states_dir='../state', states_prefix='', 
        district_dummies_dir='../ACE_districts', district_dummies_prefix='',
        save=True, load_saved=True,
        save_dir=f'/home/sgreenhill/wotus/satellite_imagery/pipeline_ready/pred/grid_{grid}'
    ) for x in folds}]

all_datasets = [ajd_dataset] + pred_datasets

dataloaders = {
    x: torch.utils.data.DataLoader(
        data.ConcatDataset(
            [d[x] for d in all_datasets]
        ),
        batch_size=batch_size,
        shuffle=True, 
        num_workers=num_cores 
    ) for x in folds
}


optimizer = optim.Adam(model.parameters(), lr=learning_rate, 
                        weight_decay=regularization)
loss_fn = nn.CrossEntropyLoss()
metrics = {'accuracy': torch_utils.accuracy}

preds = train.run_model(model, optimizer, loss_fn, dataloaders['test'],
                        metrics, save_summary_steps=1, mode='eval', 
                        targets=['ajd'],
                        testing=False)

preds_df = pd.DataFrame({k: preds[k] for k in 
                         ['pointid', 'probability_ajd', 'ajd_predictions', 'ajd_labels']})

# unlist the pointids
preds_df['pointid'] = [v[0] for v in preds['pointid']]

preds_df.to_csv(f'{model_dir}/{model_name}/test_set_predictions.csv', 
                index=False)