'''
Predict Cowardin codes on grid points.
Note that we use only 50,000 randomly sampled prediction points for this for
computational reasons, since we only want these predictions for tables.
For convenience, we pull the 50,000 points from the set of prediction data
sampled for the AJD model.
'''

import sys
import os
src_path = '/home/sgreenhill/repos/WOTUS/new_imagery/src'
sys.path.append(src_path)
os.chdir(src_path)
import torch_utils
import models
import data
import train
import torch
import torch.nn as nn
import torch.optim as optim
from torch.multiprocessing import cpu_count
import numpy as np
import pandas as pd
from tqdm import tqdm
import multiprocessing as mp

model_dir = '/home/sgreenhill/repos/WOTUS/new_imagery/analysis/cowardin_code/'
model_name = 'model_results/'

chkpt_file = model_dir + model_name + 'best.pth.tar'
model = models.resnet18(in_channels=34, n_classes=9)
learning_rate = 1e-3
regularization = 1e-4

default_cuda_device = 'cuda:3'
torch.cuda.set_device(3)
chkpt = torch_utils.load_checkpoint(chkpt_file, model, device=torch.device(default_cuda_device))

model.eval()

batch_size = 1
num_cores = int(cpu_count())

# load the data we want to predict on
pred_dir = f'/home/sgreenhill/wotus/prediction_points/'
pred_pts = pd.read_csv(f'{pred_dir}/pred_pts_sampled_50k.csv')

for rule in ['Rapanos', 'NWPR', 'CWR']:
    rule_preds_df = pd.DataFrame()
    print(f'Starting {rule}')
    for grid in range(1, 51): # np.sort(pred_pts.process_order.unique()):
        print(f'----GRID {grid}----')
        # filter to the grid 
        these_pred_pts = pred_pts.loc[pred_pts.prcss_r == grid]

        ids = these_pred_pts.pred_id.astype(str)

        # drop missings
        # corrupted files from the new NHD data
        missing_new_nhd = np.load(f'{pred_dir}/new_nhd_corrupted_points.p.npy', allow_pickle=True)
        rule_missings = np.load(f'{pred_dir}/rules/rule_drops_{rule}.npy', allow_pickle=True)

        missings = np.concatenate([rule_missings, missing_new_nhd])

        n_predrop = len(ids)
        ids = ids[~ids.isin(missings)]
        n_postdrop = len(ids)
        print(f'Dropped {n_predrop - n_postdrop} missing ids.')

        if len(ids) == 0:
            continue

        # converting back to integer identifiers plays nicer with data.py
        ids = ids.astype(int)

        names = {'predict': list(ids)}
        labels = {'predict': None}
        cowardin_labels = {'predict': None}
        resource_labels = {'predict': None}
        ajd_labels = {'predict': None}


        pred_dir = f'/home/sgreenhill/wotus/prediction_points/'

        datasets, dataloaders = data.get_NAIP_dataloaders(
            names, labels, batch_size, num_cores, 
            normalize=True, 
            normalize_path='/home/sgreenhill/repos/WOTUS/new_imagery/analysis/wotus/main_model//train/layer_mean_sd/',
            n_naip_bands=4, naip=True,
            nwi=True, nhd=True, dem=True, ecoregion=True, nlcd=True, prism=True,
            gnatsgo=True, hq_dist=True, 
            district_dummies=True, regrid=False, rule_dummies=True, 
            rule_one_hot=True, states=True,
            return_datasets=True,
            root_dir= f'{pred_dir}/grid_{grid}',
            image_dir = f'NAIP/', image_prefix='NAIP_',
            nwi_dir='NWI/', nwi_prefix='NWI_',
            nhd_dir='../NHD_new_prediction', 
            nhd_prefix='NHD_', nhd_stacked=True, dem_dir='DEM', 
            ecoregion_dir = 'Ecoregion', nlcd_dir='NLCD', prism_dir='PRISM', 
            prism_ppt_dir='PPT', gnatsgo_dir='gNATSGO', 
            dem_prefix='DEM_', ecoregion_prefix='Ecoregion_',
            gnatsgo_prefix='gNATSGO_', gnatsgo_stacked=True,
            nlcd_prefix='NLCD_', prism_prefix='PRISM_', 
            prism_ppt_prefix='PPT_', 
            hq_dist_dir='../HQ_dist/', hq_dist_prefix='',
            rule_dummies_dir=f'../rules/{rule}/', rule_dummies_prefix='',
            states_dir='../state', states_prefix='', 
            district_dummies_dir='../ACE_districts', district_dummies_prefix='',
            folds=['predict'], 
            cowardin_labels=cowardin_labels, resource_labels=resource_labels,
            ajd_labels=ajd_labels,
            shuffle=False
        )

        optimizer = optim.Adam(model.parameters(), lr=learning_rate, 
                            weight_decay=regularization)
        loss_fn = nn.CrossEntropyLoss()
        metrics = {'accuracy': torch_utils.accuracy}

        preds = train.run_model(model, optimizer, loss_fn, dataloaders['predict'],
                                metrics, save_summary_steps=1, mode='predict', testing=False,
                                targets=['cowardin'],
                                cuda_device=default_cuda_device)

        preds_df = pd.DataFrame(
                    np.concatenate(
                        preds['cowardin_probabilities']
                    )
                ).add_prefix('cowardin_probability_') 

        # unlist the pointids
        preds_df['pointid'] = [v[0] for v in preds['pointid']]
        
        # get the argmax pred
        preds_df['cowardin_prediction'] = preds['cowardin_predictions']
        
        rule_preds_df = pd.concat([rule_preds_df, preds_df])
    rule_preds_df.to_csv(f'{model_dir}/model_results/grid_predictions_{rule}.csv')