'''
Predict on test set
'''
import sys
import os
src_path = '/home/sgreenhill/repos/WOTUS/new_imagery/src'
sys.path.append(src_path)
os.chdir(src_path)
import torch_utils
import models
import data
import train
import torch
import torch.nn as nn
import torch.optim as optim
from torch.multiprocessing import cpu_count
import numpy as np
import pandas as pd
from tqdm import tqdm
import multiprocessing as mp

# load the model
model_dir = '/home/sgreenhill/repos/WOTUS/new_imagery/analysis/cowardin_code/'
model_name = 'model_results/'

chkpt_file = model_dir + model_name + 'best.pth.tar'
model = models.resnet18(in_channels=34, n_classes=9)
learning_rate = 1e-3
regularization = 1e-4

torch.cuda.set_device(0)
model.to('cuda:0')
chkpt = torch_utils.load_checkpoint(chkpt_file, model, device=torch.device('cuda:0'))

model = nn.DataParallel(model, device_ids=[0, 1, 2, 3])
model.eval()

batch_size = 1
num_cores = int(cpu_count())

path = '/home/sgreenhill/wotus'

split_path = f'{path}/misc/train_test_split_naip_only.csv'
split = pd.read_csv(split_path)
split_slug = 'naip_simon'

split = split.loc[~split.rule.isnull()]
# merge in the information for additional tasks
resource_cw = pd.read_csv(f'{path}/AJD/pointid_resourcetype_crosswalk.csv')
split = split.merge(resource_cw, on='pointid', how='inner', validate='1:1')
split = split.loc[(split.cowardin_numeric != -1) & (split.resource_numeric != -1)]

# drop pointids where data is missing
split = data.drop_missings(split, drop_S2=False)

test = split.loc[split.split_group == 'test']

names, labels, cowardin_labels, resource_labels, ajd_labels = data.make_split_dicts(split, cowardin=True)

datasets, dataloaders, data_config = data.get_NAIP_dataloaders(
    names, labels, batch_size, num_cores, 
    normalize=True, 
    normalize_path='/home/sgreenhill/repos/WOTUS/new_imagery/analysis/wotus/main_model/train/layer_mean_sd/',
    n_naip_bands=4,
    naip=True, nwi=True, 
    nhd=True, nhd_dir='../../../hdruckenmiller/NHD_new/', nhd_prefix='NHD_',
    dem=True, ecoregion=True, nlcd=True, prism=True,
    gnatsgo=True, hq_dist=True,
    district_dummies=True, regrid=False, rule_dummies=True, rule_one_hot=True,
    states=True,
    return_datasets=True, return_config=True,
    cowardin=True,
    cowardin_labels=cowardin_labels, resource_labels=resource_labels,
    ajd_labels=ajd_labels,
    folds=['test'],
    load_saved=True,
    save=True,
    save_dir=f'/home/sgreenhill/wotus/satellite_imagery/pipeline_ready/ajd_new_nhd_test_set'
)

optimizer = optim.Adam(model.parameters(), lr=learning_rate, 
                        weight_decay=regularization)
loss_fn = nn.CrossEntropyLoss()
metrics = {'accuracy': torch_utils.accuracy}

preds = train.run_model(model, optimizer, loss_fn, dataloaders['test'], metrics,
                        targets=['cowardin'],
                        save_summary_steps=1, mode='eval', 
                        testing=False)

preds_df = pd.DataFrame(
                    np.concatenate(
                        preds['cowardin_probabilities']
                    )
                ).add_prefix('cowardin_probability_') 


# unlist the pointids
preds_df['pointid'] = [v[0] for v in preds['pointid']]

# get the argmax pred
preds_df['cowardin_prediction'] = preds['cowardin_predictions']

preds_df['cowardin_labels'] = preds['cowardin_labels']

preds_df.to_csv(f'{model_dir}/{model_name}/test_set_predictions.csv', 
                index=False)