'''
Predict WOTUS on Sackett dense sample
'''

import sys
import os
src_path = '/home/sgreenhill/repos/WOTUS/new_imagery/src'
sys.path.append(src_path)
os.chdir(src_path)
import torch_utils
import models
import data
import train
import torch
import torch.nn as nn
import torch.optim as optim
from torch.multiprocessing import cpu_count
import numpy as np
import pandas as pd

rule_dict = {'1986/88': 'Rapanos', 'NWPR': 'NWPR', 'CWR': 'CWR'}

# Start w Rapanos only
rule = '1986/88'

# load the model
model_dir = '/home/sgreenhill/repos/WOTUS/new_imagery/analysis/wotus/main_model/'
model_name = 'model_results/'

chkpt_file = model_dir + model_name + 'best.pth.tar'
model = models.resnet18(in_channels=34, n_classes=2)
learning_rate = 1e-3
regularization = 1e-4

torch.cuda.set_device(2)
chkpt = torch_utils.load_checkpoint(chkpt_file, model, device=torch.device('cuda:2'))

model.eval()

batch_size = 1
num_cores = cpu_count()

# loop over rules
for rule in ['Rapanos', 'CWR', 'NWPR']:
    print(f'Starting {rule}')
    # load the data we want to predict on
    pred_dir = '/home/sgreenhill/wotus/prediction_points/'
    sackett_tiles = pd.read_csv(f'{pred_dir}/Sackett_sample_NAIP_tiles.csv')
    ids = sackett_tiles.prediction_id.str.split('_').str[-1].to_numpy()
    names = {'predict': list(ids)}
    labels = {'predict': None}
    cowardin_labels = {'predict': None}
    resource_labels = {'predict': None}
    ajd_labels = {'predict': None}

    datasets, dataloaders = data.get_NAIP_dataloaders(
        names, labels, batch_size, num_cores, n_naip_bands=4, naip=True,
        nwi=True, nhd=True, dem=True, ecoregion=True, nlcd=True, prism=True,
        gnatsgo=True, hq_dist=True, 
        district_dummies=True, regrid=False, rule_dummies=True, rule_one_hot=True,
        states=True,
        return_datasets=True,
        root_dir= pred_dir + 'Sackett_layers',
        image_dir = 'NAIP_2019', image_prefix='NAIP_Sackett_', 
        nwi_dir='NWI/', nwi_prefix='NWI_Sackett_',
        nhd_dir='../../../../hdruckenmiller/NHD_new_Sackett', nhd_prefix='NHD_Sackett_',
        dem_dir='DEM', 
        ecoregion_dir = 'Ecoregion', nlcd_dir='NLCD', prism_dir='PRISM', 
        prism_ppt_dir='PPT', gnatsgo_dir='gNATSGO', 
        dem_prefix='DEM_Sackett_', ecoregion_prefix='Ecoregion_Sackett_',
        gnatsgo_prefix='gNATSGO_Sackett_',
        nlcd_prefix='NLCD_Sackett_', prism_prefix='PRISM_Sackett_', 
        prism_ppt_prefix='PPT_Sackett_', gnatsgo_stacked=True,
        hq_dist_dir='HQ_dist/', hq_dist_prefix='Sackett_',
        rule_dummies_dir=f'rules/{rule}/', rule_dummies_prefix='Sackett_',
        states_dir='state', states_prefix='Sackett_', 
        district_dummies_dir='NAIP_ACE_districts', district_dummies_prefix='',
        folds=['predict'], cowardin_labels=cowardin_labels, resource_labels=resource_labels,
        ajd_labels=ajd_labels
    )

    optimizer = optim.Adam(model.parameters(), lr=learning_rate, 
                        weight_decay=regularization)
    loss_fn = nn.CrossEntropyLoss()
    metrics = {'accuracy': torch_utils.accuracy}

    preds = train.run_model(model, optimizer, loss_fn, dataloaders['predict'],
                            metrics, save_summary_steps=1, mode='predict', testing=False)

    preds_df = pd.DataFrame({k: preds[k] for k in ['pointid', 'probability_wotus']})

    # unlist the pointids
    preds_df['pointid'] = [v[0] for v in preds_df.pointid.values]

    preds_df.to_csv(f'{model_dir}/{model_name}/Sackett_predictions_{rule}.csv', 
                    index=False)