'''
Predict navigable waters
'''

import sys
import os
src_path = '/home/sgreenhill/repos/WOTUS/new_imagery/src'
sys.path.append(src_path)
os.chdir(src_path)
import torch_utils
import models
import data
import train
import torch
import torch.nn as nn
import torch.optim as optim
from torch.multiprocessing import cpu_count
import numpy as np
import pandas as pd
from tqdm import tqdm
import multiprocessing as mp

# load the model
model_dir = '/home/sgreenhill/repos/WOTUS/new_imagery/analysis/wotus/main_model/'
model_name = 'model_results/'

chkpt_file = model_dir + model_name + 'best.pth.tar'
model = models.resnet18(in_channels=34, n_classes=2)
learning_rate = 1e-3
regularization = 1e-4

torch.cuda.set_device(2)
model = nn.DataParallel(model)

chkpt = torch_utils.load_checkpoint(chkpt_file, model, device=torch.device('cuda:2'))

model.eval()

batch_size = 1
num_cores = cpu_count()

pred_dir = f'/home/sgreenhill/wotus/prediction_points/nav_water/'
nav_meta = pd.read_csv(f'{pred_dir}/nav_water_metadata.csv')
ids = nav_meta.prediction_id.astype(str)

missings = np.load(f'{pred_dir}/missing_pts.npy', allow_pickle=True)
n_predrop = len(ids)
ids = ids[~ids.isin(missings)]
n_postdrop = len(ids)
print(f'Dropped {n_predrop - n_postdrop} missing ids.')

# converting back to integer identifiers plays nicer with data.py
ids = ids.astype(int)

names = {'predict': list(ids)}
labels = {'predict': None}
cowardin_labels = {'predict': None}
resource_labels = {'predict': None}
ajd_labels = {'predict': None}

for rule in ['Rapanos', 'CWR', 'NWPR']:
    print(f'Starting {rule}')

    datasets, dataloaders = data.get_NAIP_dataloaders(
        names, labels, batch_size, num_cores, n_naip_bands=4, naip=True,
        nwi=True, nhd=True, dem=True, ecoregion=True, nlcd=True, prism=True,
        gnatsgo=True, hq_dist=True, 
        district_dummies=True, regrid=False, rule_dummies=True, 
        rule_one_hot=True,
        states=True,
        return_datasets=True,
        root_dir= f'{pred_dir}/',
        image_dir = f'NAIP/', image_prefix='',
        nwi_dir='NWI/', 
        nhd_dir='../../../../hdruckenmiller/NHD_new_nav_water/', 
        nhd_prefix='NHD_',
        dem_dir='DEM', 
        ecoregion_dir = 'Ecoregion', nlcd_dir='NLCD', prism_dir='PRISM', 
        prism_ppt_dir='PPT', gnatsgo_dir='gNATSGO', 
        dem_prefix='DEM_', ecoregion_prefix='Ecoregion_',
        gnatsgo_prefix='gNATSGO_',
        nlcd_prefix='NLCD_', prism_prefix='PRISM_', 
        prism_ppt_prefix='PPT_', gnatsgo_stacked=True,
        hq_dist_dir='HQ_dist/', hq_dist_prefix='',
        rule_dummies_dir=f'rules/{rule}/', rule_dummies_prefix='',
        states_dir='state', states_prefix='', 
        district_dummies_dir='ACE_districts', district_dummies_prefix='',
        folds=['predict'], cowardin_labels=cowardin_labels, 
        resource_labels=resource_labels, ajd_labels=ajd_labels,
        shuffle=False
    )

    optimizer = optim.Adam(model.parameters(), lr=learning_rate, 
                        weight_decay=regularization)
    loss_fn = nn.CrossEntropyLoss()
    metrics = {'accuracy': torch_utils.accuracy}

    preds = train.run_model(model, optimizer, loss_fn, dataloaders['predict'],
                            metrics, save_summary_steps=1, mode='predict', testing=False)

    preds_df = pd.DataFrame({k: preds[k] for k in ['pointid', 'probability_wotus']})

    # unlist the pointids
    preds_df['pointid'] = [v[0] for v in preds_df.pointid.values]

    preds_df.to_csv(f'{model_dir}/{model_name}/nav_water_predictions_{rule}.csv', 
                    index=False)

