'''
Predict on test set, looping over rules (rather than predicting only on the rule
that each AJD in the test set was decided for)
'''

import sys
import os
src_path = '/home/sgreenhill/repos/WOTUS/new_imagery/src'
sys.path.append(src_path)
os.chdir(src_path)
import torch_utils
import models
import data
import train
import torch
import torch.nn as nn
import torch.optim as optim
from torch.multiprocessing import cpu_count
import numpy as np
import pandas as pd
from tqdm import tqdm
import multiprocessing as mp

# load the model
model_dir = '/home/sgreenhill/repos/WOTUS/new_imagery/analysis/wotus/main_model/'
model_name = 'model_results/'

chkpt_file = model_dir + model_name + 'best.pth.tar'
model = models.resnet18(in_channels=34, n_classes=2)
learning_rate = 1e-3
regularization = 1e-4

torch.cuda.set_device(0)
chkpt = torch_utils.load_checkpoint(chkpt_file, model, device=torch.device('cuda:0'))

model = nn.DataParallel(model, device_ids=[0, 1, 2, 3])
model.eval()

batch_size = 1
num_cores = int(cpu_count())

path = '/home/sgreenhill/wotus'

split_path = f'{path}/misc/train_test_split_naip_only.csv'
split = pd.read_csv(split_path)
split_slug = 'naip_simon'

split = split.loc[~split.rule.isnull()]

# drop pointids where data is missing
split = data.drop_missings(split, drop_S2=False)

test = split.loc[split.split_group == 'test']

names, labels, cowardin_labels, resource_labels, ajd_labels = data.make_split_dicts(split)

for rule in ['Rapanos', 'CWR', 'NWPR']:
    print(f'--- Starting {rule} ---')
    datasets, dataloaders, data_config = data.get_NAIP_dataloaders(
        names, labels, batch_size, num_cores, 
        normalize=True,
        normalize_path=f'{model_dir}/train/layer_mean_sd',
        n_naip_bands=4, 
        naip=True, nwi=True, 
        nhd=True, nhd_dir='../../../hdruckenmiller/NHD_new/', nhd_prefix='NHD_',
        dem=True, ecoregion=True, nlcd=True, prism=True,
        gnatsgo=True, hq_dist=True,
        district_dummies=True, regrid=False, 
        rule_dummies=True, rule_one_hot=True, 
        rule_dummies_dir=f'NAIP_rule_dummies/{rule}',
        states=True,
        return_datasets=True, return_config=True, 
        cowardin_labels=cowardin_labels, resource_labels=resource_labels,
        ajd_labels=ajd_labels,
        folds=['test'],
        load_saved=False,
        save=False
    )

    optimizer = optim.Adam(model.parameters(), lr=learning_rate, 
                            weight_decay=regularization)
    loss_fn = nn.CrossEntropyLoss()
    metrics = {'accuracy': torch_utils.accuracy}

    preds = train.run_model(model, optimizer, loss_fn, dataloaders['test'],
                            metrics, save_summary_steps=1, mode='eval', 
                            testing=False)

    preds_df = pd.DataFrame({k: preds[k] for k in 
                            ['pointid', 'probability_wotus', 'predictions', 'labels']})

    # unlist the pointids
    preds_df['pointid'] = [v[0] for v in preds['pointid']]

    preds_df.to_csv(f'{model_dir}/{model_name}/test_set_predictions_{rule}.csv', 
                    index=False)