### S1 FIGURE - TRAIN/VAL/TEST SPLIT 

library(rgdal)
library(readr)
library(scales)

path <- "/Users/hannahdruckenmiller/Dropbox/Water_ArmyCorp/"

data <- read.csv(paste0(path, "dataGIS/misc_train_test_split_naip_only.csv"))

points <- read.csv(paste0(path, "dataGIS/ajds/AJD_jds_wet_dry_season_clean_v2.csv"))

data <- merge(data[,c("pointid", "split_group")], points[,c("pointid", "latitude", "longitude")])

ace <- readOGR(paste0(path, "dataRAW/geography/USACE_Regulatory_Boundary/USACE_Regulatory_Boundary.shp"))
ace <- subset(ace,!(DISTRICT %in% c("Honolulu", "Alaska")))

data$col <- "seagreen3"
data$col[data$split_group=="val"] <- "tan2"
data$col[data$split_group=="test"] <- "royalblue3"

sp <- SpatialPoints(data[,c("longitude", "latitude")])
proj4string(sp) <- proj4string(ace)

ace <- spTransform(ace, CRS("+init=epsg:3085"))
sp <- spTransform(sp, CRS("+init=epsg:3085"))

pdf(paste0(path, "drafts/FIGURES/FigS3_random.pdf"), width = 11, height = 8)
plot(ace, lwd = 0.25)
points(sp, pch = 19, cex = 0.02, col = data$col)
legend(4110104, 7117671, c("Train", "Validation", "Test"), c("seagreen3", "tan2", "royalblue3"))
dev.off()

pdf(paste0(path, "drafts/FIGURES/FigS3_valtest.pdf"), width = 11, height = 8)
plot(ace, lwd = 0.25)
points(sp[data$split_group=="train"], pch = 19, cex = 0.02, col = data$col[data$split_group=="train"])
points(sp[data$split_group!="train"], pch = 19, cex = 0.02, col = data$col[data$split_group!="train"])
legend(4110104, 7117671, c("Train", "Validation", "Test"), c("seagreen3", "tan2", "royalblue3"))
dev.off()

  

