
* set Dropbox path
global myDrop = "C:/Users/brantjwalker/Dropbox/Water_ArmyCorp"

* probability_wotus is main variable of interest
* prediction is 0 or 1 version of the probability


*==============================================================================*
*                         setup file for error analysis                        *
*==============================================================================*


* CNN predicted probabilities for all three rules
insheet using "$myDrop/results/predictions/wotus/main_model/model_results/test_set_predictions.csv", clear
ren probability_wotus prob_cnn
*ren labels ajd_decision
tempfile wotus_mlTestSet
sa "`wotus_mlTestSet.dta'", replace

* this has only a subset of the columns, and pointid
insheet using "$myDrop/dataRAW/ajds/AJD_jds_wet_dry_season_clean_v2.csv", clear
merge 1:1 pointid using "`wotus_mlTestSet.dta'", assert(1 3) keep(3) nogen
sa "`wotus_mlTestSet.dta'", replace

* this has the full AJDs with the pointid identifier
insheet using "$myDrop/dataRAW/ajds/20220531/jds202205312309.csv", clear
ren watersnames watname
* m:1 because there is one duplicate here (jdid=5184976, watname=WA)
merge m:1 jdid watname latitude longitude using "`wotus_mlTestSet.dta'", keep(3) nogen
sa "`wotus_mlTestSet.dta'", replace

* this makes loops below simpler
replace jdbasis = "Rapanos" if jdbasis == "1986/88"

g group_all = 1
g group_rapanos = jdbasis == "Rapanos"
g group_nwpr    = jdbasis == "NWPR"
g group_cwr     = jdbasis == "CWR"

g ajd_decision = wateroftheus == "Yes" if wateroftheus != ""
g       accuracy = ajd_decision == 1 if prob_cnn >= .5 & prob_cnn < .
replace accuracy = ajd_decision == 0 if prob_cnn <  .5 & prob_cnn < .

*==============================================================================*
*                              accuracy curve results                          *
*==============================================================================*


*---------------------- calculate accuracy levels and shares ------------------*

g sh_above_score_cutoffs       = .
g accuracy_above_score_cutoffs = .

* generate grid of score cutoffs: (0.50, 0.51, ..., 0.99)
g      score_cutoffs_hi = .49 + .01*_n in 1/50
assert score_cutoffs_hi >= .50 & score_cutoffs_hi < 1 in 1/50

* loop over the score cutoffs for cutoffs 0.50...0.99
forv n = 1/50 {

	* what share of validation AJDs have WOTUS-ML score above this cutoff?
	g myvar = prob_cnn > score_cutoffs_hi[`n'] if prob_cnn < .
	su myvar
	loc rmean = r(mean)
	replace sh_above_score_cutoffs = `rmean' in `n'
	drop myvar
	
	* what is accuracy of the valiation AJDs that have WOTUS-ML Score above this cutoff?
	su accuracy if prob_cnn > score_cutoffs_hi[`n'] & prob_cnn < .
	loc rmean = r(mean)
	replace accuracy_above_score_cutoffs = `rmean' in `n'
}


g sh_below_score_cutoffs = .
g accuracy_below_score_cutoffs = .


* generate grid of score cutoffs: (0.50, 0.49, ..., 0.01)
g      score_cutoffs_lo = .51 - .01*_n in 1/50
assert score_cutoffs_lo > 0 & score_cutoffs_lo <= 0.50 if score_cutoffs_lo < .

* loop over the score cutoffs for cutoffs 0.50...0.01
forv n = 1/50 {

	* what share of validation AJDs have WOTUS-ML score above this cutoff?
	g myvar = prob_cnn < score_cutoffs_lo[`n'] & prob_cnn < .
	su myvar
	loc rmean = r(mean)
	replace sh_below_score_cutoffs = `rmean' in `n'
	drop myvar
	
	* what is accuracy of the valiation AJDs that have WOTUS-ML Score below this cutoff?
	su accuracy if prob_cnn < score_cutoffs_lo[`n'] & prob_cnn < .
	loc rmean = r(mean)
	replace accuracy_below_score_cutoffs = `rmean' in `n'
	
}



*---------------------------- plot the accuracy curve -------------------------*
g xval = .
g yval = .

* which score cutoffs have >99 percent accuracy?
loc n = 1
foreach cutoff in 75 80 85 90 95 999 {
	g sh_above_score_cutoffs`cutoff'_hi = sh_above_score_cutoffs if accuracy_above_score_cutoffs > .`cutoff' & accuracy_above_score_cutoffs < .
	g sh_below_score_cutoffs`cutoff'_lo = sh_below_score_cutoffs if accuracy_below_score_cutoffs > .`cutoff' & accuracy_above_score_cutoffs < .

	* among these points that are >99% accurate, pick which has the largest share (i.e., closest to 99%)
	egen myvar_hi = max(sh_above_score_cutoffs`cutoff'_hi)
	egen myvar_lo = max(sh_below_score_cutoffs`cutoff'_lo)

	* sum the shares on the left and right tails and save it
	egen myvar = rowtotal(myvar_hi myvar_lo)
	replace yval = myvar in `n'
	
	replace xval = 0.`cutoff' in `n'

	drop sh_above_score_cutoffs`cutoff'_hi sh_below_score_cutoffs`cutoff'_lo myvar_hi myvar_lo myvar
	loc n = `n' + 1
}

*--------------------------- plot cumulative accuracy -------------------------*
* look at list to determine labels and lines
* specifically, find cutoff such that accuracy is just above 0.99, 0.95, 0.90, or 0.85
l accuracy_below_score_cutoffs score_cutoffs_lo in 1/50
l accuracy_above_score_cutoffs score_cutoffs_hi in 1/50


tw connected accuracy_below_score_cutoffs score_cutoffs_lo, graphr(color(white)) yscale(noline) msize(small) ///
	ytit("Mean left tail accuracy in test set", size(15-pt)) ///
	xtit("Predicted regulatory probability", size(15-pt)) ///
	ylab(0.99 0.95 0.90 0.85, nogrid labsize(12-pt) angle(horizontal)) ///
	xlab(0.00 0.07 0.17 0.31, nogrid labsize(12-pt)) ///
	xline(0.00 0.07 0.17 0.31, lpattern(dash) lcolor(gs12)) ///
	yline(0.99 0.95 0.90 0.85, lpattern(dash) lcolor(gs12))
graph export "$myDrop/results/figures/accuracy_curve/accuracy_lefttail.wmf", replace
graph export "$myDrop/results/figures/accuracy_curve/accuracy_lefttail.png", width(10000) replace
	
tw connected accuracy_above_score_cutoffs score_cutoffs_hi, graphr(color(white)) yscale(noline) msize(small) ///
	ytit("Mean right tail accuracy in test set", size(15-pt)) ///
	xtit("Predicted regulatory probability", size(15-pt)) ///
	ylab(0.99 0.95 0.90 0.85 0.80 0.75, nogrid labsize(12-pt) angle(horizontal)) ///
	xlab(1.00     0.95 0.83 0.73 0.63 0.52, nogrid labsize(12-pt)) ///
	xline(1.00    0.95 0.83 0.73 0.63 0.52, lpattern(dash) lcolor(gs12)) ///
	yline(0.99 0.95 0.90 0.85 0.80 0.75, lpattern(dash) lcolor(gs12))
graph export "$myDrop/results/figures/accuracy_curve/accuracy_righttail.wmf", replace
graph export "$myDrop/results/figures/accuracy_curve/accuracy_righttail.png", width(10000) replace

*---------------------------------- plot cdfs ---------------------------------*
* x-axis values here come from the 'plot cumulative accuracy' graphs above
* and show threshold WOTUS-ML scores

* look at list to determine labels and lines
l sh_below_score_cutoffs score_cutoffs_lo in 1/50
l sh_above_score_cutoffs score_cutoffs_hi in 1/50

tw connected sh_below_score_cutoffs score_cutoffs_lo, graphr(color(white)) yscale(noline) msize(small) ///
	ytit("Cumulative left tail share of test set", size(15-pt)) ///
	xtit("Predicted regulatory probability", size(15-pt)) ///
	xlab( 0.00 0.07 0.17 0.31, nogrid labsize(12-pt)) ///
	xline(0.00 0.07 0.17 0.31, lpattern(dash) lcolor(gs12)) ///
	ylab( 0.00 0.24 0.41 0.58, nogrid labsize(12-pt) angle(horizontal)) ///
	yline(0.00 0.24 0.41 0.58, lpattern(dash) lcolor(gs12)) 
graph export "$myDrop/results/figures/accuracy_curve/cdf_lefttail.wmf", replace
graph export "$myDrop/results/figures/accuracy_curve/cdf_lefttail.png", width(10000) replace
	
tw connected sh_above_score_cutoffs score_cutoffs_hi, graphr(color(white)) yscale(noline) msize(small) ///
	ytit("Cumulative right tail share of test set", size(15-pt)) ///
	xtit("Predicted regulatory probability", size(15-pt)) ///
	xlab( 0.95 0.83 0.73 0.63 0.52, nogrid labsize(12-pt)) ///
	xline(0.95 0.83 0.73 0.63 0.52, lpattern(dash) lcolor(gs12)) ///
	ylab( 0.02 0.11 0.16 0.22 0.28, nogrid labsize(12-pt) angle(horizontal)) ///
	yline(0.02 0.11 0.16 0.22 0.28, lpattern(dash) lcolor(gs12)) 
graph export "$myDrop/results/figures/accuracy_curve/cdf_righttail.wmf", replace
graph export "$myDrop/results/figures/accuracy_curve/cdf_righttail.png", width(10000) replace

	
	
* generate file for other figures
export delimited using "$myDrop/results/predictions/wotus/main_model/model_results/testSetWithAJDinfo.csv", replace
	
